-------------------------------------------------------------------------------
-- ElvUI Chat Tweaks By Lockslap (US, Bleeding Hollow)
-- <Borderline Amazing>, http://ba-guild.com
-- Based on functionality provided by Prat and/or Chatter
-------------------------------------------------------------------------------
local Module	= ElvUI_ChatTweaks:NewModule("Raid Helper", "AceEvent-3.0", "AceConsole-3.0")
local L			= LibStub("AceLocale-3.0"):GetLocale("ElvUI_ChatTweaks")
Module.name		= L["Raid Helper"]

local format = string.format

local db, options
local defaults = {
	profile = {
		refreshment	= true,
		feast		= true,
		cauldron	= true,
		repair		= true,
	}
}

-- strings
local prepared	= L["%s has prepared a %s."]
local putDown	= L["%s has put down a %s."]

-- spell ids for everything
local feasts = {
	57426,	-- Fish Feast (WOTLK)
	87643,	-- Broiled Dragon Feast
	87915,	-- Goblin BBQ Feast
	87644	-- Seafood Magnifique Feast
}
local cauldrons = {
	92649,	-- Cauldron of Battle
	92712	-- Big Cauldron of Battle
}
local repairBots = {
	67826,	-- Jeeves
	22700,	-- Field Repair Bot 74A
	44389,	-- Field Repair Bot 110G
	54711	-- Scrapbot
}
local ids = {
	-- repair bots
	["67826"]	= "49040",	-- Jeeves
	["22700"]	= "18232",	-- Field Repair Bot 74A
	["44389"]	= "34113",	-- Field Repair Bot 110G
	["54711"]	= "40769",	-- Scrapbot
	
	-- cauldrons
	["92649"]	= "62288",	-- Cauldron of Battle
	["92712"]	= "65460",	-- Big Cauldron of Battle
	
	-- feasts
	["57426"]	= "43015",	-- Fish Feast (WOTLK)
	["87643"]	= "62289",	-- Broiled Dragon Feast
	["87915"]	= "60858",	-- Goblin BBQ Feast
	["87644"]	= "62290"	-- Seafood Magnifique Feast
}

local function InArray(array, needle)
	if #array == 0 then return false end
	for _, value in pairs(array) do
		if needle == value then return true end
	end
	return false
end

local function GetChat()
	if GetNumRaidMembers() > 0 then
		return (IsRaidLeader() or IsRaidOfficer()) and "RAID_WARNING" or "RAID"
	elseif GetNumPartyMembers() > 0 then
		return "PARTY"
	end
	return "SAY"
end

local function InGroup()
	return (GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0) and true or false
end

function Module:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, subEvent, hideCaster, srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags, spellID, spellName, spellSchool, extraSpellID, extraSpellName, extraSpellSchool, auraType)
	if not InGroup() or InCombatLockdown() or not subEvent or not spellID or not srcName or not spellName then return end
	
	if subEvent == "SPELL_CAST_START" then
		-- feasts/cauldrons
		if InArray(feasts, spellID) or InArray(cauldrons, spellID) then
			local _, link = GetItemInfo(ids[tostring(spellID)])
			SendChatMessage(format(prepared, srcName, link), GetChat(), nil)
		end
	elseif subEvent == "SPELL_CAST_SUCCESS" then
		-- repair bot or refreshment table
		if spellID == 43987 then
			SendChatMessage(format(putDown, srcName, "refreshment table"), GetChat(), nil)
		elseif InArray(repairBots, spellID) then
			local _, link = GetItemInfo(ids[tostring(spellID)])
			SendChatMessage(format(putDown, srcName, link), GetChat(), nil)
		end
	end
end

function Module:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function Module:OnDisable()
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function Module:OnInitialize()
	self.db = ElvUI_ChatTweaks.db:RegisterNamespace("RaidHelper", defaults)
	db = self.db.profile
end

function Module:Info()
	return L["Notifies the raid/party when someone places down a feast, cauldron, or repair bot."]
end

function Module:GetOptions()
	if not options then
		options = {
			refreshment = {
				type	= "toggle",
				name	= L["Ritual of Refreshment"],
				desc	= L["Announce when a mage begins casting Ritual of Refreshment."],
				get		= function() return db.refreshment end,
				set		= function(_, value) db.refreshment = value end,
			},
			feast = {
				type	= "toggle",
				name	= L["Feasts"],
				desc	= L["Announce when someone puts down a feast."],
				get		= function() return db.feast end,
				set		= function(_, value) db.feast = value end,
			},
			cauldron = {
				type	= "toggle",
				name	= L["Cauldron of Battle"],
				desc	= L["Announce when an alchemist puts down a (Big) Cauldron of Battle."],
				get		= function() return db.cauldron end,
				set		= function(_, value) db.cauldron = value end,
			},
			repair = {
				type	= "toggle",
				name	= L["Repair Bot"],
				desc	= L["Announce when an engineer puts down a repair bot."],
				get		= function() return db.repair end,
				set		= function(_, value) db.repair = value end,
			}
		}
	end
	return options
end